
/*============================================================================
  File:     MAXTypes.sql

  Summary:  This script code is used to test the code demos in
            the Transact-SQL or SQLCLR presentation by Bob Beauchemin

  Date:     Nov 15, 2005

  SQL Server Version: 9.00.1399.06
------------------------------------------------------------------------------
  Copyright (C) 2005 Bob Beauchemin, SQLskills, Inc.
  All rights reserved.

  This script is intended only as a supplement to demos and lectures
  given by Bob Beauchemin.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

use tempdb
go

-- Rows can hold more than a page
-- in SQL Server 2005
CREATE TABLE largeVariableValues (
 column1 VARCHAR(8000),
 column2 VARCHAR(8000)
)
GO

CREATE TABLE largeVariableValues2 (
 column1 VARCHAR(MAX),
 column2 VARCHAR(MAX)
)
GO

-- Fixed length fields may not span more than a page
CREATE TABLE largeFixedValues (
 column1 CHAR(80),
 column2 CHAR(8000) )  -- error
GO

-- VARCHAR(MAX) IS NOT the same data type as VARCHAR
-- only 8000
declare @a varchar(max)
set @a = replicate('a', 8500)
print datalength(@a)

-- nope
declare @a varchar(max)
set @a = replicate(cast('a' as varchar(8000)), 8500)
print datalength(@a)

-- this gets them all
declare @a varchar(max)
set @a = replicate(cast('a' as varchar(max)), 8500)
print datalength(@a)


-- WRITE method

declare @x nvarchar(max)
declare @y nvarchar(max)

set @x = replicate(cast('a' as nvarchar(max)), 5000)
print datalength(@x)

set @y = replicate(cast('a' as nvarchar(max)), 600)

-- note that REPLICATE and WRITE takes number of characters
SET @x.WRITE(@y, 5000, 1000)
-- DATALENGTH is in bytes
print datalength(@x)

SET @x.WRITE(@y, 5200, 1000)
print datalength(@x)

SET @x.WRITE(@y, 8700, 1000)
print datalength(@x)



DECLARE @x nvarchar(max)
DECLARE @y nvarchar(max)

SET @y = replicate(cast('a' as nvarchar(max)), 500)
SET @x.WRITE(@y, 0, 500) -- error, can't write to NULL instance
SET @x = replicate(cast('a' as nvarchar(max)), 5000)
PRINT len(@x)        -- prints 5000
SET @x.WRITE(@y, 5000, 500)
PRINT len(@x)        -- prints 5500
SET @x.WRITE(@y, 5000, 995) -- not an error, appends 500
PRINT len(@x)        -- prints 5500
SET @x.WRITE(@y, 5200, 500) -- not an error, replaces 300, appends 200
PRINT len(@x)        -- prints 5700
SET @x.WRITE(@y, 0, 500) -- error, can't write to NULL instance